pub struct FunctionDefBody<'a> {
pub statements: Vec<Statement<'a>>,
pub src: &'a str,
}Expand description
Parsed body of a stored function, with an offset for mapping spans back to the outer source file.
Fields
statements: Vec<Statement<'a>>Parsed statements from the function body
src: &'a strThe body source string (borrowed from the outer source)